@@ -215,7 +215,11 @@ static void rugged_repo_new_with_backend(git_repository **repo, VALUE rb_path, V
 	if (error) goto cleanup;
 
 	error = git_odb_add_backend(odb, odb_backend, 1);
-	if (error) goto cleanup;
+	if (error) {
+		assert(odb_backend->free);
+		odb_backend->free(odb_backend);
+		goto cleanup;
+	}
 
 	error = git_repository_wrap_odb(repo, odb);
 	if (error) goto cleanup;
@@ -224,7 +228,11 @@ static void rugged_repo_new_with_backend(git_repository **repo, VALUE rb_path, V
 	if (error) goto cleanup;
 
 	error = backend->refdb_backend(&refdb_backend, backend, path);
-	if (error) goto cleanup;
+	if (error) {
+		assert(refdb_backend->free);
+		refdb_backend->free(refdb_backend);
+		goto cleanup;
+	}
 
 	error = git_refdb_set_backend(refdb, refdb_backend);
 	if (error) goto cleanup;
@@ -248,9 +256,6 @@ cleanup:
 	git_odb_free(odb);
 	git_refdb_free(refdb);
 
-	if (odb_backend != NULL) odb_backend->free(odb_backend);
-	if (refdb_backend != NULL) refdb_backend->free(refdb_backend);
-
 	rugged_exception_check(error);
 }
 
@@ -824,6 +829,62 @@ static VALUE rb_git_repo_merge_analysis(int argc, VALUE *argv, VALUE self)
 	return result;
 }
 
+/*
+ *  call-seq:
+ *    repo.revert_commit(revert_commit, our_commit, options = {}) -> index
+ *    
+ *	Reverts the given commit against the given "our" commit, producing an
+ *	index that reflects the result of the revert.
+ */
+static VALUE rb_git_repo_revert_commit(int argc, VALUE *argv, VALUE self)
+{
+	VALUE rb_revert_commit, rb_our_commit, rb_options;
+	git_commit *revert_commit, *our_commit;
+	git_index *index;
+	git_repository *repo;
+	git_merge_options opts = GIT_MERGE_OPTIONS_INIT;
+	unsigned int mainline = 0;
+	int error;
+
+	rb_scan_args(argc, argv, "20:", &rb_revert_commit, &rb_our_commit, &rb_options);
+
+	if (TYPE(rb_revert_commit) == T_STRING)
+		rb_revert_commit = rugged_object_rev_parse(self, rb_revert_commit, 1);
+
+	if (TYPE(rb_our_commit) == T_STRING)
+		rb_our_commit = rugged_object_rev_parse(self, rb_our_commit, 1);
+
+	if (!rb_obj_is_kind_of(rb_revert_commit, rb_cRuggedCommit) ||
+		!rb_obj_is_kind_of(rb_our_commit, rb_cRuggedCommit)) {
+		rb_raise(rb_eArgError, "Expected a Rugged::Commit.");
+	}
+
+	if (!NIL_P(rb_options)) {
+		VALUE rb_mainline;
+
+		Check_Type(rb_options, T_HASH);
+		rugged_parse_merge_options(&opts, rb_options);
+
+		rb_mainline = rb_hash_aref(rb_options, CSTR2SYM("mainline"));
+		if (!NIL_P(rb_mainline)) {
+			Check_Type(rb_mainline, T_FIXNUM);
+			mainline = FIX2UINT(rb_mainline);
+		}
+	}
+
+	Data_Get_Struct(self, git_repository, repo);
+	Data_Get_Struct(rb_revert_commit, git_commit, revert_commit);
+	Data_Get_Struct(rb_our_commit, git_commit, our_commit);
+
+	error = git_revert_commit(&index, repo, revert_commit, our_commit, mainline, &opts);
+	if (error == GIT_EMERGECONFLICT)
+		return Qnil;
+
+	rugged_exception_check(error);
+
+	return rugged_index_new(rb_cRuggedIndex, self, index);
+}
+
 /*
  *  call-seq:
  *    repo.merge_commits(our_commit, their_commit, options = {}) -> index
@@ -841,6 +902,7 @@ static VALUE rb_git_repo_merge_commits(int argc, VALUE *argv, VALUE self)
 	git_index *index;
 	git_repository *repo;
 	git_merge_options opts = GIT_MERGE_OPTIONS_INIT;
+	int error;
 
 	rb_scan_args(argc, argv, "20:", &rb_our_commit, &rb_their_commit, &rb_options);
 
@@ -869,7 +931,11 @@ static VALUE rb_git_repo_merge_commits(int argc, VALUE *argv, VALUE self)
 	Data_Get_Struct(rb_our_commit, git_commit, our_commit);
 	Data_Get_Struct(rb_their_commit, git_commit, their_commit);
 
-	rugged_exception_check(git_merge_commits(&index, repo, our_commit, their_commit, &opts));
+	error = git_merge_commits(&index, repo, our_commit, their_commit, &opts);
+	if (error == GIT_EMERGECONFLICT)
+		return Qnil;
+
+	rugged_exception_check(error);
 
 	return rugged_index_new(rb_cRuggedIndex, self, index);
 }
@@ -1868,7 +1934,7 @@ static int rugged__checkout_notify_cb(
 /**
  * The caller has to free the returned git_checkout_options paths strings array.
  */
-static void rugged_parse_checkout_options(git_checkout_options *opts, VALUE rb_options)
+void rugged_parse_checkout_options(git_checkout_options *opts, VALUE rb_options)
 {
 	VALUE rb_value;
 
@@ -2464,6 +2530,61 @@ static VALUE rb_git_repo_cherrypick(int argc, VALUE *argv, VALUE self)
 	return Qnil;
 }
 
+/*
+ *  call-seq:
+ *    repo.cherrypick_commit(commit, our_commit, [mainline, options]) -> nil
+ *
+ *  Cherry-pick the given commit on the given base in-memory and
+ *  return an index with the result.
+ *
+ *  `commit` can be either a string containing a commit id or a
+ *  `Rugged::Commit` object.
+ *
+ *  `our_commit` is the base commit, can be either a string containing
+ *  a commit id or a `Rugged::Commit` object.
+ *
+ *  `mainline` when cherry-picking a merge, this is the parent number
+ *   (starting from 1) which should be considered the mainline.
+ */
+static VALUE rb_git_repo_cherrypick_commit(int argc, VALUE *argv, VALUE self)
+{
+	VALUE rb_options, rb_commit, rb_our_commit, rb_mainline;
+
+	git_repository *repo;
+	git_commit *commit, *our_commit;
+	git_merge_options opts = GIT_MERGE_OPTIONS_INIT;
+	git_index *index;
+	int error, mainline;
+
+	rb_scan_args(argc, argv, "21:", &rb_commit, &rb_our_commit, &rb_mainline, &rb_options);
+
+	if (TYPE(rb_commit) == T_STRING) {
+		rb_commit = rugged_object_rev_parse(self, rb_commit, 1);
+	}
+	if (TYPE(rb_our_commit) == T_STRING) {
+		rb_our_commit = rugged_object_rev_parse(self, rb_our_commit, 1);
+	}
+
+	if (!rb_obj_is_kind_of(rb_commit, rb_cRuggedCommit)) {
+		rb_raise(rb_eArgError, "Expected a Rugged::Commit.");
+	}
+	if (!rb_obj_is_kind_of(rb_our_commit, rb_cRuggedCommit)) {
+		rb_raise(rb_eArgError, "Expected a Rugged::Commit.");
+	}
+
+	Data_Get_Struct(self, git_repository, repo);
+	Data_Get_Struct(rb_commit, git_commit, commit);
+	Data_Get_Struct(rb_our_commit, git_commit, our_commit);
+
+	rugged_parse_merge_options(&opts, rb_options);
+
+	mainline = NIL_P(rb_mainline) ? 0 : FIX2UINT(rb_mainline);
+	error = git_cherrypick_commit(&index, repo, commit, our_commit, mainline, &opts);
+	rugged_exception_check(error);
+
+	return rugged_index_new(rb_cRuggedIndex, self, index);
+}
+
 void Init_rugged_repo(void)
 {
 	id_call = rb_intern("call");
@@ -2518,6 +2639,8 @@ void Init_rugged_repo(void)
 	rb_define_method(rb_cRuggedRepo, "merge_analysis", rb_git_repo_merge_analysis, -1);
 	rb_define_method(rb_cRuggedRepo, "merge_commits", rb_git_repo_merge_commits, -1);
 
+	rb_define_method(rb_cRuggedRepo, "revert_commit", rb_git_repo_revert_commit, -1);
+
 	rb_define_method(rb_cRuggedRepo, "path_ignored?", rb_git_repo_is_path_ignored, 1);
 
 	rb_define_method(rb_cRuggedRepo, "reset", rb_git_repo_reset, 2);
@@ -2535,6 +2658,7 @@ void Init_rugged_repo(void)
 	rb_define_method(rb_cRuggedRepo, "checkout_head", rb_git_checkout_head, -1);
 
 	rb_define_method(rb_cRuggedRepo, "cherrypick", rb_git_repo_cherrypick, -1);
+	rb_define_method(rb_cRuggedRepo, "cherrypick_commit", rb_git_repo_cherrypick_commit, -1);
 	rb_define_method(rb_cRuggedRepo, "fetch_attributes", rb_git_repo_attributes, -1);
 
 	rb_cRuggedOdbObject = rb_define_class_under(rb_mRugged, "OdbObject", rb_cObject);
